clearvars -except timerS own_machine_performance; clc; close all;

%% PARAMETERS

% preferences
risk_aver   = 2;
beta        = 0.95;
r           = 0.03;
R = 1+ r;

%filename to store "true" values
fname= 'disc_true_dist.txt';

agname= 'disc_agrid.txt'; %name of the file containing the asset grid points
igname1= 'disc_igrid1.txt'; %name of the file containing the asset grid points
igname2= 'disc_igrid2.txt'; %name of the file containing the asset grid points
aggconsname= 'agg_c_disc.txt'; %name of the file to store the "true" aggregate consumption

% cash on hand / savings grid
na          = 10000;
xmax        = 50; 
xgrid_par   = 0.5; %1 for linear, 0 for L-shaped
borrow_lim  = 0;

% computation
maxit    = 1000;
crit    = 1.0e-6;
Nsim        = 50000;
Tsim        = 500;

% income risk: AR(1) + IID in logs
nyP         = 9; % persistent component
rho_logyP   = 0.95;
sd_logyP    = 0.01;

nyT         = 5; %transitory component (not a state variable)
sd_logyT   = sqrt(0.04*(1-rho_logyP^2));

%% DRAW RANDOM NUMBERS
rng(2020);
yPrand = rand(Nsim,Tsim);
yTrand = randn(Nsim,Tsim);

%% SET UP GRIDS

%persistent income
[logyPgrid, yPtrans, yPdist] = rouwenhorst(nyP, -0.5*sd_logyP^2, sd_logyP, rho_logyP);

yPgrid = exp(logyPgrid);
yPcumdist = cumsum(yPdist);
yPcumtrans = cumsum(yPtrans,2);

% transitory income: disretize normal distribution
if nyT>1
    width = fzero(@(x)discrete_normal(nyT,-0.5*sd_logyT^2 ,sd_logyT ,x),2);
    [temp,logyTgrid,yTdist] = discrete_normal(nyT,-0.5*sd_logyT^2 ,sd_logyT ,width);

elseif nyT==1
    logyTgrid = 0;
    yTdist = 1;
end
yTgrid = exp(logyTgrid);

%gross labor income grid;
ygrid = zeros(nyP,nyT);
ydist= zeros(nyP,nyT);
for iyP = 1:nyP
    for iyT= 1:nyT
        ygrid(iyP,iyT) = yPgrid(iyP)*yTgrid(iyT);
        ydist(iyP,iyT) = yPdist(iyP)*yTdist(iyT);
    end
end    


%% ASSET GRIDS

% cash on hand grids: different min points for each value of (iyP)
for iyP = 1:nyP
    xgrid(:,iyP) = linspace(0,1,na)'.^(1/xgrid_par);
    xgrid(:,iyP) = borrow_lim + min(ygrid(iyP,:)) + (xmax-borrow_lim).*xgrid(:,iyP);
end

% savings grid for EGP
ns = na;
sgrid = linspace(0,1,na)';
sgrid = sgrid.^(1/xgrid_par);
sgrid = borrow_lim + (xmax-borrow_lim).*sgrid;

%% UTILITY FUNCTION

if risk_aver==1
    u = @(c)log(c);
else    
    u = @(c)(c.^(1-risk_aver)-1)./(1-risk_aver);
end    

u1 = @(c) c.^(-risk_aver);
u1inv = @(u) u.^(-1./risk_aver);

%% INITIALIZE CONSUMPTION FUNCTION
conguess = r.*xgrid;

%% ITERATE ON EULER EQUATION WITH ENDOGENOUS GRID POINTS

con = conguess;

iter = 0;
cdiff = 1000;
tic;
while iter <= maxit && cdiff>crit
    iter = iter + 1;
    sav = zeros(na,nyP);
    emuc= zeros(ns,nyP);

    conlast = con;
    muc  = u1(conlast);   %muc on grid for x' 

    %create interpolating function for each yP'
    for iy = 1:nyP
        coninterp{iy} = griddedInterpolant(xgrid(:,iy),conlast(:,iy),'linear');
    end


    % loop over current persistent income
    for iy = 1:nyP

        %loop over future income realizations    
        for iyP2 = 1:nyP
            for iyT2 = 1:nyT
                emuc(:,iy) = emuc(:,iy) + u1(coninterp{iyP2}(R.*sgrid(:) + yPgrid(iyP2)*yTgrid(iyT2))) * yPtrans(iy,iyP2) * yTdist(iyT2);
            end
        end

        muc1(:,iy) = beta.*R.*emuc(:,iy);
        con1(:,iy) = u1inv(muc1(:,iy));
        cash1(:,iy) = con1(:,iy) + sgrid;


        % loop over current period cash on hand
        for ix = 1:na 
            if xgrid(ix,iy)<cash1(1,iy) %borrowing constraint binds
                sav(ix,iy) = borrow_lim;
            else %borrowing constraint does not bind;
                sav(ix,iy) = lininterp1(cash1(:,iy),sgrid,xgrid(ix,iy));
            end                
        end
        con(:,iy) = xgrid(:,iy) - sav(:,iy);
    end   

    cdiff = max(max(abs(con-conlast)));
    %disp(['Iteration no. ' int2str(iter), ' max con fn diff is ' num2str(cdiff)]);
end    
duration=toc;

%% SIMULATE
tic;
yPindsim = zeros(Nsim,Tsim);
logyTsim = zeros(Nsim,Tsim);
logyPsim = zeros(Nsim,Tsim);
ysim = zeros(Nsim,Tsim);    
xsim = zeros(Nsim,Tsim);
ssim = zeros(Nsim,Tsim);

%create interpolating function
for iy = 1:nyP
    savinterp{iy} = griddedInterpolant(xgrid(:,iy),sav(:,iy),'linear');
end

%initialize permanent income;
it = 1;
yPindsim(yPrand(:,it)<= yPcumdist(1),it) = 1;
for iy = 2:nyP
    yPindsim(yPrand(:,it)> yPcumdist(iy-1) & yPrand(:,it)<=yPcumdist(iy),it) = iy;
end

%loop over time periods
for it = 1:Tsim
    if mod(it,100) ==0
        %disp([' Simulating, time period ' int2str(it)]);
    end

    %permanent income realization
    if it > 1
        yPindsim(yPrand(:,it)<= yPcumtrans( yPindsim(:,it-1),1),it) = 1;
        for iy = 2:nyP
            yPindsim(yPrand(:,it)> yPcumtrans(yPindsim(:,it-1),iy-1) & yPrand(:,it)<=yPcumtrans(yPindsim(:,it-1),iy),it) = iy;
        end
    end
    logyPsim(:,it) = logyPgrid(yPindsim(:,it));

    %transitory income realization
    logyTsim(:,it) = - 0.5*sd_logyT^2 + yTrand(:,it).*sd_logyT;

    ysim(:,it) = exp(logyPsim(:,it) + logyTsim(:,it));

    %update cash on hand
    if it>1
        xsim(:,it) = R.*ssim(:,it-1) + ysim(:,it);
    end

    %savings choice
    if it<Tsim        
        for iy = 1:nyP
            ssim(yPindsim(:,it)==iy,it) = savinterp{iy}(xsim(yPindsim(:,it)==iy,it));
        end
    end
    ssim(ssim<borrow_lim) = borrow_lim;


end

duration_dist = toc;
%convert to assets
asim = (xsim - ysim)./R;

%consumption
csim = xsim - ssim;

duration_agg = duration + duration_dist;
%% Aggregate consumption
C = mean(csim(:,Tsim));

%% Write true values to file
true_vals = con;
p = '../Data/';
fname=strcat(p, fname);
csvwrite(fname,true_vals);

aggconsname = strcat(p, aggconsname);
csvwrite(aggconsname,[C;duration; duration_agg]);

aa = repmat(sgrid, 1, 9);
zz1 = repmat(yPgrid, 1,10000)';
zz2 = repmat((sum(ygrid'.*ydist')')./ sum(ydist')', 1, 10000)';

agname = strcat(p,agname);
igname1= strcat(p, igname1);
igname2= strcat(p, igname2);

csvwrite(agname,aa);
csvwrite(igname1,zz1);
csvwrite(igname2,zz2);